'use strict';

exports.__esModule = true;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Properties that will trigger an updateNbVisibleRows.
 *
 * @type {string[]}
 */
var REQUIRE_UPDATE_PROPS = ['rowHeight', 'minRowsDisplayed', 'onNbVisibleRowsChange'];

var AutoresizeTable = function (_Component) {
  _inherits(AutoresizeTable, _Component);

  function AutoresizeTable() {
    var _temp, _this, _ret;

    _classCallCheck(this, AutoresizeTable);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, _Component.call.apply(_Component, [this].concat(args))), _this), _this.onResize = function () {
      _this.updateNbVisibleRows();
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  AutoresizeTable.prototype.componentDidMount = function componentDidMount() {
    window.addEventListener('resize', this.onResize);
    this.updateNbVisibleRows();
  };

  AutoresizeTable.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    var _this2 = this;

    if (REQUIRE_UPDATE_PROPS.some(function (prop) {
      return nextProps[prop] !== _this2.props[prop];
    })) {
      this.updateNbVisibleRows(nextProps);
    }
  };

  AutoresizeTable.prototype.componentWillUnmount = function componentWillUnmount() {
    window.removeEventListener('resize', this.onResize);
  };

  /**
   * Called when the window is resized.
   */

  /**
   * Update the number of visible rows.
   *
   * @param {object} props
   */

  AutoresizeTable.prototype.updateNbVisibleRows = function updateNbVisibleRows() {
    var props = arguments.length <= 0 || arguments[0] === undefined ? this.props : arguments[0];

    if (!props.onNbVisibleRowsChange) return;

    var container = this.getContainer();
    var rowHeight = this.getRowHeight();
    var tbody = this.getTbody();
    var availableHeight = container.offsetHeight - tbody.offsetTop;

    var nbVisibleRows = Math.floor(availableHeight / rowHeight);
    nbVisibleRows = nbVisibleRows <= props.minRowsDisplayed ? props.minRowsDisplayed : nbVisibleRows;

    props.onNbVisibleRowsChange(nbVisibleRows);
  };

  /**
   * Returns container of table.
   *
   * @returns {Element}
   */


  AutoresizeTable.prototype.getContainer = function getContainer() {
    return this.props.container || this.getTable().parentElement;
  };

  /**
   * Returns table element.
   *
   * @returns {Element}
   */


  AutoresizeTable.prototype.getTable = function getTable() {
    var domElement = _reactDom2.default.findDOMNode(this);

    if (domElement.tagName.toLowerCase() !== 'table') throw new Error('Your child must be of type table');

    return domElement;
  };

  /**
   * Returns tbody element.
   *
   * @returns {Element}
   */


  AutoresizeTable.prototype.getTbody = function getTbody() {
    var tbody = this.getTable().getElementsByTagName('tbody')[0];

    if (!tbody) throw new Error('Unable to find tbody in table');

    return tbody;
  };

  /**
   * Returns first row element.
   *
   * @returns {Element}
   */


  AutoresizeTable.prototype.getFirstRow = function getFirstRow() {
    var firstRow = this.getTbody().children[0];

    if (!firstRow) throw new Error('Unable to find first row in tbody, ' + 'specify rowHeight prop instead');

    return firstRow;
  };

  /**
   * Returns line height.
   *
   * @returns {number}
   */


  AutoresizeTable.prototype.getRowHeight = function getRowHeight() {
    if (this.props.rowHeight) return this.props.rowHeight;

    var firstRow = this.getFirstRow();
    return firstRow.offsetHeight;
  };

  AutoresizeTable.prototype.render = function render() {
    if (_react2.default.Children.count(this.props.children) !== 1) throw new Error('You must have only one child of type table');

    return this.props.children;
  };

  return AutoresizeTable;
}(_react.Component);

AutoresizeTable.propTypes = {
  children: _react.PropTypes.node,
  container: typeof Element === 'undefined' ? _react.PropTypes.object : _react.PropTypes.object,
  minRowsDisplayed: _react.PropTypes.number,
  onNbVisibleRowsChange: _react.PropTypes.func,
  rowHeight: _react.PropTypes.number
};
AutoresizeTable.defaultProps = {
  minRowsDisplayed: 1
};
exports.default = AutoresizeTable;